import logging
from pathlib import Path
from pyflowdroid._cli_tools import _run_command, cli_header
from pyflowdroid.consts import (
    PYFLOWDROID_PATH,
    DEFAULT_APK_FOLDER_NAME,
    ANDROID_FOLDER_NAME,
    FLOWDROID_EXEC_NAME,
)


def _save_logs(logs: str, log_path: str):
    """
    Save a log file.

    Parameters
    ----------
    logs : str
        Content of the file.
    log_path : str
        Path to log file.
    """
    with open(log_path, "w") as f:
        f.write(logs)


def analyze_apk(path: str, sources_and_sinks: str = "", save_logs: bool = True) -> str:
    """
    Execute FlowDroid analysis in an APK file.

    Parameters
    ----------
    path : str
        Path-like object pointing to the apk file.
    sources_and_sinks : path, optional
        Defines the path to the sources and sinks file. As part of this
        library there are two sources_and_sinks files: 'large.txt' and
        'small.txt'. Both can be passed as valid values of this param.
        If not specified, the default sources and sinks file is 'small.txt'.
        A path to a custom sources and sinks file can be passed as a string too.
    save_logs : bool, optional
        Defines whether or not save raw logs from FlowDroid, by default True

    Returns
    -------
    str
        Logs with the raw output of FlowDroid analyzer.

    Raises
    ------
    ValueError
        If `path` does not exist or does not point to an apk file.
    ValueError
        If `sources_and_sinks` is passed and does not point to a existing file
        or is not one of default values ('small.txt' or 'large.txt').
    """

    # Convert path to pathlib object
    apk_path = Path(path)

    # Check if path exists, is a file and its extension is .apk
    if apk_path.exists() and apk_path.is_file() and str(path).endswith(".apk"):

        # Create the path for required resources
        fd_path = Path(PYFLOWDROID_PATH, FLOWDROID_EXEC_NAME)
        android_path = Path(PYFLOWDROID_PATH, ANDROID_FOLDER_NAME)
        sns_path = Path(sources_and_sinks)

        # If no valid path specified, use default sources and sinks file
        if not (sns_path.exists() and sns_path.is_file()):
            if sources_and_sinks == "large.txt":
                sns_path = Path(PYFLOWDROID_PATH, "sources_sinks", "large.txt")
            elif sources_and_sinks in ["small.txt", ""]:
                sns_path = Path(PYFLOWDROID_PATH, "sources_sinks", "small.txt")
            else:
                raise ValueError("Invalid sources and sinks file path.")

        # Log the sources and sinks file being used
        logging.info(f"Using sources and sinks from '{sns_path}'")

        # Create the flowdroid call for the given apk file
        command = f"java -jar {fd_path} -a {apk_path} -p {android_path} -s {sns_path}"

        # Execute command and get the output
        logging.info(f"Analyzing '{apk_path}'")
        logs = _run_command(command)

        # Save logs if save_logs is True
        if save_logs:
            log_path = apk_path.with_suffix(".log")
            logging.info(f"Flowdroid logs saved in {log_path}")
            _save_logs(logs, log_path)

        # Return the logs
        return logs

    # Raise execption for invalid paths
    else:
        raise ValueError(f"Address {path} does not point to a valid apk file")


def analyze_apk_folder(
    path: str = DEFAULT_APK_FOLDER_NAME,
    sources_and_sinks: str = "",
    save_logs: bool = True,
) -> dict:
    """
    Execute FlowDroid analysis in all APK files contained in a folder.

    Parameters
    ----------
    path : str, optional
        Path to the folder. By default, the default folder is 'apks'.
    sources_and_sinks : path, optional
        Defines the path to the sources and sinks file. As part of this
        library there are two sources_and_sinks files: 'large.txt' and
        'small.txt'. Both can be passed as valid values of this param.
        If not specified, the default sources and sinks file is 'small.txt'.
        A path to a custom sources and sinks file can be passed as a string too.
    save_logs : bool, optional
        Defines whether or not save raw logs from FlowDroid, by default True

    Returns
    -------
    dict
        Pairs of APK file name and its raw output of FlowDroid analyzer.

    Raises
    ------
    ValueError
        If `path` does not exist or does not point to a folder.
    ValueError
        If `sources_and_sinks` is passed and does not point to a existing file
        or is not one of default values ('small.txt' or 'large.txt').
    """

    # Create apk folder if it does not exist
    if path == DEFAULT_APK_FOLDER_NAME:
        Path(path).mkdir(parents=True, exist_ok=True)

    # Create the path object with the folder
    folder_path = Path(path)
    logging.info(f"Analyzing '{folder_path}'")

    # Check if path exists and is a folder
    if folder_path.exists() and folder_path.is_dir():

        # Retreive all apk files in the folder
        apk_paths = [
            str(apk_path)
            for apk_path in folder_path.glob("**/*.apk")
            if apk_path.is_file()
        ]
        logging.info(f"Found {len(apk_paths)} apks in '{folder_path}'")

        # Analize all apks and store their logs
        return {
            str(apk_path): analyze_apk(apk_path, sources_and_sinks, save_logs)
            for apk_path in apk_paths
        }

    # Raise execption for invalid paths
    raise ValueError(f"Address {path} does not point to a valid folder")


def analyze(
    path: str = DEFAULT_APK_FOLDER_NAME,
    sources_and_sinks: str = "",
    save_logs: bool = True,
) -> tuple[int, int, list]:
    """
    Execute FlowDroid analysis in the given path.

    Parameters
    ----------
    path : str, optional
        Path pointing the target of the analysis. If path points to an apk file,
        it returns the results of its analysis. If path points to a folder, it
        returns the overal results of analysing all apk files in that folder.
        By default, 'apks'.
    sources_and_sinks : path, optional
        Defines the path to the sources and sinks file. As part of this
        library there are two sources_and_sinks files: 'large.txt' and
        'small.txt'. Both can be passed as valid values of this param.
        If not specified, the default sources and sinks file is 'small.txt'.
        A path to a custom sources and sinks file can be passed as a string too.
    save_logs : bool, optional
        Defines whether or not save raw logs from FlowDroid, by default True

    Returns
    -------
    tuple[int, int, list]
        Tuple containing the total number of apks analized, the total number of
        leaks found and a list of the leaky apks.

    Raises
    ------
    ValueError
        If `path` does not exist.
    ValueError
        If `sources_and_sinks` is passed and does not point to a existing file
        or is not one of default values ('small.txt' or 'large.txt').
    """

    # Create the path object with the folder
    path = Path(path)

    # Check if path exists and is a folder
    if path.exists():
        if path.is_dir():
            logs = analyze_apk_folder(path, sources_and_sinks, save_logs)
        else:
            logs = {path: analyze_apk(path, sources_and_sinks, save_logs)}
        return quantify_leaks(logs)

    # Raise execption for invalid paths
    raise ValueError(f"Address {path} does not point to an existing location")


def quantify_leaks(logs: dict) -> tuple[int, int, list]:
    """
    Quantify the number of leaks in all the analyzed APK.

    Parameters
    ----------
    logs : dict
        Pairs of APK file name and its raw output of FlowDroid analyzer.

    Returns
    -------
    tuple[int, int, list]
        Tuple containing the total number of apks analized, the total number of
        leaks found and a list of the leaky apks.

    Raises
    ------
    ValueError
        If `logs` is not a dictionary.
    """

    # Check if logs is a dictionary
    if not isinstance(logs, dict):
        raise ValueError("logs must be a dictionary")

    leaky_apks = []
    total_leaks = 0
    total_apps = len(logs.keys())
    for apk, log in logs.items():
        leak_count = count_leaks_in_log_file(log)
        if leak_count != 0:
            total_leaks += leak_count
            leaky_apks.append(apk)

    return total_apps, total_leaks, leaky_apks


def count_leaks_in_log_file(log: str) -> int:
    """
    Count the number of leaks in the given logs.

    Parameters
    ----------
    logs : str
        Raw Flowdroid output

    Returns
    -------
    int
        Total number of leaks found by Flowdroid
    """

    # Parse the total count of leaks found in the logs
    # TODO: Improve this with regex
    last_line = log.split("\n")[-1]
    leaks = last_line.split()[-2]
    return int(leaks) if leaks.isdigit() else 0


def generate_report(total_apps, total_leaks, leaky_apks) -> str:
    """
    Generate a report of the analysis giving quantitative parameters.

    Parameters
    ----------
    total_apps : _type_
        Total number of apks analized
    total_leaks : _type_
        Total number of leaks found by Flowdroid
    leaky_apks : _type_
        List of the leaky apks.

    Returns
    -------
    str
        Comprehensive report of the analysis conducted
    """

    # Generate the report
    report = cli_header("PYFLOWDROID REPORT")
    report += f"Analized: {total_apps}\n"
    report += f"Leaks found: {total_leaks}\n\n"

    if len(leaky_apks):
        report += f"Leaky apks:\n"

        for lap in leaky_apks:
            report += f" - {lap}\n"
    else:
        report += "No leaky apks found\n"

    return report



import logging
from pyflowdroid.download import fetch
from pyflowdroid.install import install_deps
from pyflowdroid.analyze import analyze, analyze_apk, generate_report

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s [%(levelname)s] %(message)s',
    datefmt='%H:%M:%S'
)

__all__ = [
    'fetch',
    'install_deps',
    'analyze',
    'analyze_apk',
    'generate_report',
]

__version__ = '0.2.0'


import typer
import pyflowdroid


app = typer.Typer()


@app.command()
def install():
    pyflowdroid.install_deps()

@app.command()
def analyze(path: str):
    total, leaks, leaky_apps = pyflowdroid.analyze(path)
    typer.echo(pyflowdroid.generate_report(total, leaks, leaky_apps))

@app.command()
def download(amount:int, path: str, provider: str):
    pyflowdroid.fetch(amount, provider, path)
    typer.echo(f"Downloaded {amount} apks from {provider}")
    
app()


import abc
import shutil
import urllib
import logging
import functools
from pathlib import Path
from urllib.request import urlopen
from bs4 import BeautifulSoup
from pyflowdroid.consts import DEFAULT_APK_PROVIDER


class ApkProvider(metaclass=abc.ABCMeta):
    """
    Abstract class to define the structure of an APK provider.

    Parameters
    ----------
    force_redownload : bool, optional
        Defines wheter or not the provider will overwrite existing apk files
        when is trying to download a file with a name already existing in the
        give folder, by default False.
    """

    def __init__(self, force_redownload: bool = False):
        self.force_redownload = force_redownload

    @abc.abstractmethod
    def avialable_apks(self) -> dict:
        """
        Abstract method to be implemented by each apk provider. In classes
        inheriting from ApkProvider, this method is expected to return a
        dictionary with the data from the apks the provider has avialable.

        The expected format of the dictionary is:
        {
            'apk1.apk': 'http://url/to/apk1.apk',
            'another.apk': 'http://url/to/another.apk',
            'verynice.apk': 'http://url/to/verynice.apk'
        }

        Returns
        -------
        dict
            Dictionary with the data of the apks avialable.
        """
        ...

    def download_apk(self, apk_name: str, apk_url: str, path: str) -> None:
        """
        Download an apk from a given url.

        Parameters
        ----------
        apk_name : str
            Name of the apk to be downloaded.
        apk_url : str
            Url of the apk to be downloaded.
        path : str
            Path where the apk will be downloaded.

        Raises
        ------
        ValueError
            If path does not points to a folder
        """

        # Check if path is a folder and exists
        folder_path = Path(path)
        if not folder_path.exists() or folder_path.is_file():
            raise ValueError(f"{path} does not points to a folder")

        # Generate the path to the apk file that is going to be downloaded
        apk_path = folder_path / apk_name

        # Check if the apk file already exists and if force_redownload is False
        if apk_path.exists() and not self.force_redownload:

            # If the apk file already exists and force_redownload is False skips
            # the download
            logging.info(f"{apk_name} already exists")
            return

        # Download the apk file from the given url
        try:
            logging.info(f"Downloading {apk_name} from {apk_url}")
            hdr = {"User-Agent": "Mozilla/5.0"}
            req = urllib.request.Request(apk_url, headers=hdr)
            with urllib.request.urlopen(req) as response, open(
                apk_path, "wb"
            ) as out_file:
                shutil.copyfileobj(response, out_file)
        # Notify if the apk file could not be downloaded
        except urllib.error.HTTPError:
            logging.error(f"Error downloading {apk_name} from {apk_url}")

    def download_apks(self, amount: int, path: str, create_path: bool = True) -> None:
        """
        Download a given amount of apks from the provider.

        Parameters
        ----------
        amount : int
            Amount of apks to be downloaded.
        path : str
            Path where the apks will be downloaded.
        create_path : bool, optional
            Defines wheter or not the path will be created if it does not exists.
        """

        # Create the path if it does not exists
        if create_path:
            Path(path).mkdir(parents=True, exist_ok=True)

        # Fetch the dictionary with all the apks from the provider
        avialable_apks = self.avialable_apks()

        # Download each apk until reaching the amount of apks to be downloaded
        for i, apk_name in enumerate(avialable_apks):
            if i == amount:
                logging.info(f"Downloaded {i} apks")
                return
            apk_url = avialable_apks[apk_name]
            self.download_apk(apk_name, apk_url, path)

        # Download each apk until reaching the amount of apks to be downloaded
        logging.error(
            f"Downloaded {i} apks instead of {amount}. No more apks to download"
        )


class CubapkProvider(ApkProvider):
    """
    CubapkProvider class to download apks from the cubapk.com website.
    """

    def __init__(self, force_redownload: bool = False):
        super().__init__(force_redownload)
        self.base_url = "https://cubapk.com"

    @functools.lru_cache(maxsize=None)
    def avialable_apks(self) -> dict:
        """
        Generates a dictionary with the avialable apks from cubapk.com website.
        """

        # Empty dictionary to store the avialable apks
        apk_dict = {}

        # Interation variable to control the page we are parsing in the website
        page = 1

        # If there is another page to be parsed
        while True:
            page_url = f"{self.base_url}/store/?page={page}"

            # Fetches the website raw HTML
            try:
                request = urllib.request.Request(
                    page_url, headers={"User-Agent": "Mozilla/5.0"}
                )
                text = urlopen(request).read()
            except urllib.error.HTTPError:
                logging.info(f"Index of {self.base_url} has {page-1} pages")
                break

            # Parses the website
            soup = BeautifulSoup(text, features="lxml")
            data = soup.findAll("div", attrs={"class": "app-data"})
            for div in data:

                # Parse the apk name
                raw_name = div.find("div", attrs={"class": "app-title"}).text
                apk_name = "".join(filter(str.isalnum, raw_name))
                apk_name += ".apk"

                # Parse the apk url
                download_div = div.find("div", attrs={"class": "app-meta"})
                raw_link = download_div.a["href"]
                link = f"{self.base_url}{raw_link}"

                # Apend the data from the apk to the dictionary
                apk_dict[apk_name] = link

            # Updates the page number
            page += 1
        return apk_dict


# Dictionary with all the available providers
_providers = {"cubapk.com": CubapkProvider}


def get_provider(name: str, force_redownload:bool=False) -> ApkProvider:
    """
    Returns an ApkProvider instance with the given name. If there are no
    matches with the given name, the default provider is returned.

    Parameters
    ----------
    name : str
        Name of the provider to be returned.
    force_redownload : bool, optional
        Defines wheter or not the apks will be redownloaded if exist.

    Returns
    -------
    ApkProvider
        Provider instance with the given name.
    """

    if name not in _providers.keys():
        logging.warning(f"Provider '{name}' not found, using '{DEFAULT_APK_PROVIDER}'")
        return _providers[DEFAULT_APK_PROVIDER](force_redownload)
    return _providers[name](force_redownload)


def fetch(
    amount: int,
    provider: str,
    path: str,
    create_path: bool = True,
    force_redownload: bool = False,
) -> None:
    """
    Download a given amount of apks from the given provider.

    Parameters
    ----------
    amount : int
        Amount of apks to be downloaded.
    provider : str
        Name of the provider to be used.
    path : str
        Path where the apks will be downloaded.
    create_path : bool, optional
        Defines wheter or not the path will be created if it does not exists.
    force_redownload : bool, optional
        Defines wheter or not the apks will be redownloaded if they already
        exists.
    """

    # Get the provider
    provider = get_provider(provider, force_redownload)

    # Download the apks
    provider.download_apks(amount, path, create_path)


import pathlib

# Paths and File names
PYFLOWDROID_PATH = pathlib.Path(__file__).parent.resolve()
DEFAULT_APK_FOLDER_NAME = "apks"
ANDROID_FOLDER_NAME = "android-platforms-master"
FLOWDROID_EXEC_NAME = "soot-infoflow-cmd-2.9.0-jar-with-dependencies.jar"

# URLs
FLOWDROID_DOWNLOAD_URL = f"""https://github.com/secure-software-engineering/FlowDroid/releases/download/v2.9/{FLOWDROID_EXEC_NAME}"""
SABLE_DOWNLOAD_URL = """https://github.com/Sable/android-platforms/archive/master.zip"""

# OTHERS
DEFAULT_APK_PROVIDER = 'cubapk.com'


import os
import logging
import zipfile
import urllib.request
from pathlib import Path

from pyflowdroid.consts import (
    FLOWDROID_DOWNLOAD_URL,
    SABLE_DOWNLOAD_URL,
    DEFAULT_APK_FOLDER_NAME,
    FLOWDROID_EXEC_NAME,
    PYFLOWDROID_PATH,
)
from pyflowdroid._cli_tools import ProgressBar


def download_flowdroid():
    """
    Fetches Flowdroid from the internet.
    """

    # Create a path object for the Flowdroid executable
    flowdroid_path = PYFLOWDROID_PATH / FLOWDROID_EXEC_NAME

    # Download executable from GitHub
    logging.info("Downloading Flowdroid")
    urllib.request.urlretrieve(FLOWDROID_DOWNLOAD_URL, flowdroid_path, ProgressBar())


def download_android():
    """
    Fetches Android from the internet.
    """

    # Download project from GitHub
    github_zip = "master.zip"
    logging.info("Downloading Android")
    urllib.request.urlretrieve(SABLE_DOWNLOAD_URL, github_zip, ProgressBar())

    # Extract project
    logging.info("Uncompressing Android")
    with zipfile.ZipFile(github_zip, "r") as zip_ref:
        zip_ref.extractall(PYFLOWDROID_PATH)
    os.remove(github_zip)


def create_apk_folder():
    """
    Creates the APK folder.
    """

    # Create a path object for the apk folder
    logging.info("Creating Directories")
    apk_path = Path(DEFAULT_APK_FOLDER_NAME)

    # Create the folder if the folder does not exist
    apk_path.mkdir(parents=True, exist_ok=True)


def install_deps():
    """
    Installs Flowdroid and Android.
    """
    download_flowdroid()
    download_android()
    logging.info("Instalation Complete")



import random
import progressbar
import subprocess


class ProgressBar:
    """
    Progress bar call back that can be used with urlib.requests.urlretreive.

    Base code taken from:
    https://stackoverflow.com/questions/37748105/how-to-use-progressbar-module-with-urlretrieve
    """

    def __init__(self):
        self.pbar = None
        self.progress = 0

    def _update_progress_bar(self, downloaded, total_size):

        # Checks if the download is not finished and updates the progress bar
        # with the current progress
        if downloaded < total_size:
            self.pbar.update(downloaded)

        # In case is finished, finishes the progress bar
        else:
            self.pbar.finish()

    def __call__(self, block_num, block_size, total_size):

        # Define max_val according to the total size of the file if it's known
        max_val = total_size if total_size > 0 else 100

        # Create a progress bar in the first call
        if not self.pbar:
            self.pbar = progressbar.ProgressBar(maxval=max_val)
            self.pbar.start()

        # Update the progress if we know the total download size
        if total_size > 0:
            self.progress = block_num * block_size

        # Fake the progress if we don't know the total download size
        else:
            fake_remaining = max_val - self.progress
            self.progress = random.random() * fake_remaining / 3

        # Updates the status of the progress bar
        self._update_progress_bar(self.progress, max_val)


def cli_header(msg: str, char: str = "#", lenght: int = 80) -> str:
    """
    Returns a header with the given message.

    Parameters
    ----------
    msg : str
        Message that is going to be printed.
    char : str
        Character that is going to be used to fill the header.
    lenght : int
        Lenght of the header.


    Returns
    -------
    str
        Header.
    """
    
    lp = int(0.5 * (lenght - 2 - len(msg)))  # left padding
    rp = lenght - len(msg) - lp - 2  # right padding
    return f'{"#" * 80}\n{"#"+" " * rp}{msg}{" " * lp + "#"}\n{"#" * lenght}\n'


def _run_command(cmd: str) -> str:
    """
    Runs a command in a subprocess and returns the output.

    Parameters
    ----------
    cmd : str
        Command that is going to be executed.

    Returns
    -------
    str
        Output returned by the command.
    """
    return subprocess.getoutput(cmd)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pyflowdroid']

package_data = \
{'': ['*'], 'pyflowdroid': ['sources_sinks/*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'lxml>=4.8.0,<5.0.0',
 'progressbar>=2.5,<3.0',
 'typer>=0.4.1,<0.5.0']

setup_kwargs = {
    'name': 'pyflowdroid',
    'version': '0.2.0',
    'description': 'Python wrapper to use FlowDroid APK analyzer.',
    'long_description': '# PyFlowDroid\n\nPython wrappers for [FlowDroid](https://github.com/secure-software-engineering/FlowDroid) \nApk analyzer. This project was built with two goals in mind:\n\n1. Automate the creation of a FlowDroid environment out-of-the-box\n2. Allow the usage of FlowDroid from Python code \n\n## 1. Installation\n\n### 1.1 Prerequisites \n\nMake sure you have:\n\n1. A working version of [java](https://www.java.com/en/download/help/download_options.html) \n   in the PATH of your environment.\n2. A working version of [python](https://www.python.org/downloads/) in the PATH \n   of your environment.\n3. A working version of [git](https://git-scm.com/downloads) in the PATH of your \n   environment.\n\n### 1.2 Installing pyFlowDroid\n\nYou can get it from pypi using [pip](https://pip.pypa.io/en/stable/installation/):\n\n```\n$ pip install pyflowdroid\n```\n\nThen, you need to run an additional command to make pyflowdroid download and \ninstall FlowDroid and the required resources to use it:\n\n```\n$ python -m pyflowdroid install\n```\n\n## 2. Using pyflowdroid as a Python library\n\nYou can use this script as a guide for downloading and analyzing apk files\nwith pyflowdroid:\n\n```python\nimport pyflowdroid\n\n# Path to a folder where the apks are stored\napkfolder = "./apks"\n\n# If you need test apks, you can dowload the desired amount\n# from a given provider into a destination folder\npyflowdroid.fetch(10, \'cubapk.com\', apkfolder)\n\n# Analyze all the apks in a folder\napk_count, leaks_count, leaky_apps = pyflowdroid.analyze(apkfolder)\n\n# Analyze a single apk\nflowdroid_logs = pyflowdroid.analyze_apk(\'./apks/test.apk\')\n```\n\n## 3. Using pyflowdroid as a command line tool\n\nThe main advantage of using pyflowdroid as a command line tool over using\nFlowDroid directly is the automatic gathering of resources required to \nexecute the flow analysis. pyflowdroid comes bundled with all the required \nresources to allow a quick analysis of apk files.\n\nTo analyze an .apk file with the default pyflowdroid setup, just run:\n\n```\n$ python -m pyflowdroid analyze path/to/file.apk\n```\n\nSimilarly, you can perform a flow analysis on all the apks inside a folder:\n\n```\n$ python -m pyflowdroid analyze path/to/folder/\n```\n\nThis should store raw FlowDroid logs for each analyzed apk and then show a \ngeneral report like the following:\n\n```\n################################################################################\n#                              PYFLOWDROID REPORT                              #\n################################################################################\nAnalized: 5\nLeaks found: 2\n\nLeaky apps:\n - \'path/to/folder/app1.apk\'\n - \'path/to/folder/app3.apk\'\n```\n\nIf you want to get some test apks, pyflowdroid includes a download function\nto fetch apks from a given provider. Current available providers are:\n\n- [cubapk.com](https://cubapk.com/)\n\nTo fetch apks from a provider, just run:\n\n```\n$ python -m pyflowdroid download amount path/to/store/apks/ provider_name\n```\n\nFor instance, to download 10 apk files from cubapk.com, run:\n\n```\n$ python -m pyflowdroid download 10 ./myapks/ cubapk.com\n```\n\n\n## 4. Contributing to pyflowdroid\n\nIf you want to add any features to pyflowdroid you will need to get a \ndevelopment enviroment.\n\n### 4.1 Fetching the project source code\n\nYou can clone the github repository by executing:\n\n```\n$ git clone https://github.com/gvieralopez/pyFlowDroid\n$ cd pyFlowDroid\n```\n\n### 4.2 Installing pyFlowDroid dependencies\n\nYou can install them with [poetry](https://python-poetry.org/docs/#installation)\n by executing:\n\n```\n$ poetry shell\n$ poetry install\n$ poetry build\n```\n### 4.3 Download FlowDroid and its dependencies\n\nThis step will download and install FlowDroid. After doing this you can use \nFlowDroid with or without pyFlowDroid wrappers.\nSimply run:\n\n```\n$ python -m pyflowdroid install\n```\n\n### 4.4 Making your changes appear in the project\n\nJust make a Pull Request.\n\n### 4.5 Quality Assurance:\n\n#### Run tests:\n\n```\n$ pytest\n```\n\n#### Type checking:\n\n```\n$ mypy\n```\n\n#### Code style:\n\n```\n$ flake8\n```\n\n### 4.6 Pending features\n\n- Improve cli interface with hints on the parameters\n- Improve cli interface with parameters all supported API functions\n- Recognize in runtime when Flowdroid and depencies were not installed\n- Add new apk providers\n- Write pytest unit tests\n- Write documentation',
    'author': 'Gustavo Viera López',
    'author_email': 'gvieralopez@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gvieralopez/pyFlowDroid',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
